package com.compomics.proteocloud.db.accessor;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;

/**
 * Created by IntelliJ IDEA.
 * User: Lennart Martens, Thilo Muth
 * Date: 03.09.2010
 * Time: 14:26:15
 * To change this template use File | Settings | File Templates.
 */
public class Spectrum extends SpectrumTableAccessor {
	
	
    /**
     * Calls the super class.
     * @param params
     */
    public Spectrum(HashMap params){
        super(params);
    }
    
    /**
     * This constructor reads the spectrum file from a resultset. The ResultSet should be positioned such that a single
     * row can be read directly (i.e., without calling the 'next()' method on the ResultSet).
     *
     * @param aRS ResultSet to read the data from.
     * @throws SQLException when reading the ResultSet failed.
     */
    public Spectrum(ResultSet aRS) throws SQLException {
        super(aRS);
    }
    
    /**
     * This method will find a spectrum file from the current connection, based on the spectrum name.
     *
     * @param spectrumName String with the spectrum name of the spectrum file to find.
     * @param aConn     Connection to read the spectrum File from.
     * @return Spectrumfile with the data.
     * @throws SQLException when the retrieval did not succeed.
     */
    public static Spectrum findFromSpectrumName(String spectrumName, Connection aConn, boolean omssa) throws SQLException {
    	String formatted = "";
    	if(omssa){
    		formatted = spectrumName.replace("\\\\", "/");
    	} else {
    		formatted = spectrumName.replace('\\', '/');
    	}
        Spectrum temp = null;
        // Only get the last 1500 records
        PreparedStatement ps = aConn.prepareStatement("select * from spectrum where spectrumname = ? order by creationdate LIMIT 1500");
        ps.setString(1, formatted);
        ResultSet rs = ps.executeQuery();
        int counter = 0;
        while (rs.next()) {
            counter++;
            temp = new Spectrum(rs);
        }
        rs.close();
        ps.close();
        if (counter < 1) {
            throw new SQLException("Select based on spectrum name '" + formatted + "' resulted in " + counter + " results!");
        }

        return temp;
    }

    
    /**
     * This method will find a spectrum file from the current connection, based on the filename.
     *
     * @param filename String with the filename of the spectrum file to find.
     * @param aConn     Connection to read the spectrum File from.
     * @return Spectrumfile with the data.
     * @throws SQLException when the retrieval did not succeed.
     */
    public static Spectrum findFromFilename(String filename, Connection aConn) throws SQLException {
    	
        Spectrum temp = null;
        PreparedStatement ps = aConn.prepareStatement("select * from spectrum where filename = ? order by creationdate LIMIT 1500");
        ps.setString(1, filename);
        ResultSet rs = ps.executeQuery();
        int counter = 0;
        while (rs.next()) {
            counter++;
            temp = new Spectrum(rs);
        }
        rs.close();
        ps.close();
        if (counter < 1) {
            throw new SQLException("Select based on filename'" + filename + "' resulted in " + counter + " results!");
        }

        return temp;
    }
    
    /**
     * This method will find a spectrum file from the current connection, based on the spectrum filename.
     *
     * @param filename String with the filename of the spectrum file to find.
     * @param aConn     Connection to read the spectrum File from.
     * @return Spectrumfile with the data.
     * @throws SQLException when the retrieval did not succeed.
     */
    public static void checkDuplicateFile(String filename, Connection aConn) throws SQLException {
    	
        Spectrum temp = null;
        PreparedStatement ps = aConn.prepareStatement("select * from spectrum");        
        ResultSet rs = ps.executeQuery();
        int counter = 0;
        while (rs.next()) {
            counter++;
            temp = new Spectrum(rs);
            if(temp.getFilename().equals(filename.substring(0, filename.length() - 4) + "_1.mgf")) {
            	throw new SQLException("File '" + filename + "' already exists in the DB!");
            }
        }
        rs.close();
        ps.close();        
    }


    /**
     * This method will find a spectrum file from the current connection, based on the specified spectrumid.
     *
     * @param aSpectrumID long with the spectrumid of the spectrum file to find.
     * @param aConn           Connection to read the spectrum File from.
     * @return Spectrumfile with the data.
     * @throws SQLException when the retrieval did not succeed.
     */
    public static Spectrum findFromID(long aSpectrumID, Connection aConn) throws SQLException {
        Spectrum temp = null;
        PreparedStatement ps = aConn.prepareStatement(getBasicSelect() + " where spectrumid = ?");
        ps.setLong(1, aSpectrumID);
        ResultSet rs = ps.executeQuery();
        int counter = 0;
        while (rs.next()) {
            counter++;
            temp = new Spectrum(rs);
        }
        rs.close();
        ps.close();
        if (counter != 1) {
            SQLException sqe = new SQLException("Select based on spectrum ID '" + aSpectrumID + "' resulted in " + counter + " results instead of 1!");
            sqe.printStackTrace();
            throw sqe;
        }
        return temp;
    }
    
}
